import os
import json
import logging

name = "magv"

class magv_config:
    def __init__(self):
        self.logger = logging.getLogger("MANGROVE")
        self.config_json = os.path.expanduser("~/.magv/config.json")
        self.config_path = os.path.expanduser("~/.magv")
        self.repo_path = os.path.expanduser("~/.magv/magv-index")
        self.log_path = os.path.expanduser("~/.magv/magv.log")
        try:
            FIRST_RUN = False
            if not os.path.isfile(self.config_json):
                os.makedirs(self.config_path, exist_ok = True)
                os.system(f"cp {os.path.join(os.path.dirname(__file__), "config_sample.json")} {self.config_json}")
                FIRST_RUN = True
            logging.basicConfig(filename=os.path.expanduser("~/.magv/magv.log"), level=logging.DEBUG, filemode="w+")
            with open(self.config_json) as f:
                self.config = json.loads(f.read())
                f.close()
                if FIRST_RUN:
                    os.system(f"git clone {self.config['Index-Repo']} {self.repo_path}")
                    print("WARNING: YOU SHOULD CHANGE YOUR CONFIGURE FILE MANUALLY AT ~/.magv/config.json TO USE MANGROVE")
                    self.logger.warning("WARNING: YOU SHOULD CHANGE YOUR CONFIGURE FILE MANUALLY AT ~/.magv/config.json TO USE MANGROVE")
                    exit(1)
                else:
                    current_working_dir = os.getcwd()
                    os.chdir(self.repo_path)
                    os.system(f"git pull")
                    os.chdir(current_working_dir)
        except:
            self.logger.fatal("An error occured while initialization")
            os.abort()


import os
import requests
import shutil
import json

PGXN_API = "https://api.pgxn.org"
def download(path, extension, version, config):
    obj = requests.get(PGXN_API + f"/dist/{extension}/{version}/{extension}-{version}.zip")
    config.logger.info(f"Downloaded {extension} from PGXN, HTTP status code {obj.status_code}")
    if not obj.status_code == 200:
        raise Exception(obj.status_code)
    try:
        with open(os.path.join(path, f"{extension}-{version}.zip"), "wb+") as f:
            f.write(obj.content)
        f.close()
    except Exception as e:
        config.logger.error(f"Cannot write into {os.path.join(path, f'{extension}-{version}.zip')}")
        raise e
    try:
        shutil.unpack_archive(os.path.join(path, f"{extension}-{version}.zip"), path)
        os.system(f"mv {os.path.join(path, f'{extension}-{version}')}/* {path}")
        os.removedirs(os.path.join(path, f"{extension}-{version}"))
        os.remove(os.path.join(path, f.name))
    except Exception as e:
        config.logger.error(f"An error occured while performing file operations")
        raise e

def search(ext, config):
    try:
        res = json.loads(requests.get(f"https://api.pgxn.org/search/extensions?q={ext}").content)["hits"]
        ext_list = list()
        for item in res:
            ext_list.append([item["extension"], "PGXN", item["abstract"]])
        return ext_list
    except Exception as e:
        config.logger.error("An error occured while searching extension in PGXN")
        raise e


import os
import argparse
from tabulate import tabulate
import magv.default as default
import magv.pgxn as pgxn
import magv.install as install
from magv import magv_config

def search(ext, config):
    try:
        k = pgxn.search(ext, config) + default.search(ext, config)
        return k
    except Exception:
        config.logger.error(f"Failed to download {ext}")
        os.abort()

def download(path, ext, ver, source, config):
    if ver == "latest":
        ver = "master"
    try:
        if source == "MANGROVE":
            default.download(path, ext, ver, config)
        elif k[i][1] == "PGXN":
            pgxn.download(path, ext, ver, config)
    except Exception:
        config.logger.error(f"Failed to download {ext}")
        os.abort()

if __name__ == "__main__":
    config = magv_config()
    parser = argparse.ArgumentParser(description = "MANGROVE - PostgreSQL Extension Network Client")
    parser.add_argument('-s', '--search', nargs = 1, help="Search for an extension", metavar = ("extension"))
    parser.add_argument('-d', '--download', nargs = 1, help="Download an extension", metavar = ("extension"))
    parser.add_argument('-i', '--install', nargs = 1, help="Install an extension", metavar = ("extension"))
    parser.add_argument('-p', '--path', nargs = 1, help = "Specify the installtion source / Download destination", metavar = "path")

    arg = parser.parse_args()
    path_  = arg.path

    if not arg.search == None:
        print(tabulate(search(arg.search[0], config), headers=['Extension', 'Source', 'Description'], showindex="always"))

    if not arg.download == None:
        if path_ == None:
            path = os.path.join(config.config_path, arg.download[0])
        else:
            path = path_[0]
        try:
            os.makedirs(path, exist_ok = True)
        except:
            config.logger.error(f"Failed to create directories at {path}")
            os.abort()
        k = search(arg.download[0], config)
        print(tabulate(k, headers=['Extension', 'Source', 'Description'], showindex="always"))
        i = int(input(f"Which extension to download? [0 ~ {len(k)- 1}]"))
        j = input(f"Which version then?")
        download(path, k[i][0], j, k[i][1], config)

    if not arg.install == None:
        if path_ == None:
            path = os.path.join(config.config_path, arg.install[0])
        else:
            path = path_[0]
        k = search(arg.install[0], config)
        print(tabulate(k, headers=['Extension', 'Source', 'Description'], showindex="always"))
        i = int(input(f"Which extension to install? [0 ~ {len(k)- 1}]"))
        j = input("Which version then? (specific version / latest)")
        try:
            os.makedirs(path, exist_ok = True)
        except:
            config.logger.error(f"Failed to create directories at {path}")
            os.abort()
        download(path, k[i][0], j, k[i][1], config)
        try:
            install.install(path, config)
        except:
            os.abort()


import os

def install(path, config):
    if not os.getuid() == 0:
        config.logger.warning("WARNING: You are trying to install an extension without sudo")
    try:
        os.chdir(path)
        if os.path.isfile("autogen.sh"):
            if not os.system("sh ./autogen.sh") == 0:
                config.logger.error("An error occured when running autogen.sh")
                raise Exception
        if os.path.isfile("configure"):
            if not os.system("./configure") == 0:
                config.logger.error("An error occured when running configure")
                raise Exception
        if not os.system("make") == 0:
            config.logger.error("An error occured when running making")
            raise Exception
        if not os.system("make install") == 0:
            config.logger.error("An error occured when running making install")
            raise Exception
    except Exception as e:
        config.logger.error("Installation failed")


import magv.gitrepo as gitrepo
import json
import os

def download(path, ext, ver, config):
    try:
        with open(os.path.join(config.repo_path, f"{ext}.json"), "r") as f:
            repo = json.loads(f.read())["repo"]
            f.close()
    except Exception:
        config.logger.error("Extension not found")
        raise Exception("Extension not found")
    try:
        gitrepo.download(path, repo, ver, config)
    except Exception as e:
        config.logger.error("An error occured while downloading extension")
        raise e

def search(ext, config):
    try:
        dist = os.listdir(config.repo_path)
        res = list()
        for i in dist:
            if not i.find(ext) == -1:
                with open(os.path.join(config.repo_path, i), "r") as f:
                    item = json.loads(f.read())
                    f.close()
                    res.append([i.rsplit(".", 1)[0], "MANGROVE", item["abstract"]]) # Strip the ".json" part
    except Exception as e:
        config.logger.error("An error occured while searching extension")
        raise e
    return res


import os

def download(path, extension, ver, config):
    try:
        os.chdir(path)
        config.logger.info(f"Downloading git repository from {extension} to {path}, cloning from branch {ver}")
        if not os.system("git clone --depth=1 --branch " + ver + " " + extension + " " + path) == 0:
            raise Exception(f"Unable to clone git repository from")
    except Exception as e:
        config.logger.error(f"An error occured while downloading git repository")
        raise e